import importlib
import os

import stoqlib.api
from stoqlib.database.testsuite import bootstrap_suite
from stoqlib.lib.configparser import StoqConfig, register_config
from stoqlib.lib.pluginmanager import get_plugin_manager


def _install_plugin(name):
    plugin_module_name, plugin_cls_name = name.rsplit(".", maxsplit=1)
    plugin_module = importlib.import_module(plugin_module_name)
    plugin_cls = getattr(plugin_module, plugin_cls_name)

    plugin_name = plugin_cls.name
    manager = get_plugin_manager()
    plugin_dir = os.path.dirname(plugin_module.__file__)
    plugin_package_name = os.path.basename(plugin_dir)
    desc_filename = os.path.join(plugin_package_name, "{}.plugin".format(plugin_name))
    manager.register_plugin_description(desc_filename)

    if plugin_name not in manager.installed_plugins_names:
        with stoqlib.api.new_store() as store:
            manager.install_plugin(store, plugin_name)

    if plugin_name not in manager.active_plugins_names:
        manager.activate_plugin(plugin_name)


def _to_falsy(value):
    if value in ('0', '', 'None', 'False', 'false', 'f'):
        return False
    return bool(value)


def _setup_test_environment(request):
    if request.config.getvalue("skip_env_setup"):
        return

    config = StoqConfig()
    config.load_default()
    register_config(config)

    hostname = os.environ.get("STOQLIB_TEST_HOSTNAME")
    dbname = os.environ.get("STOQLIB_TEST_DBNAME")
    username = os.environ.get("STOQLIB_TEST_USERNAME")
    password = os.environ.get("STOQLIB_TEST_PASSWORD")
    port = int(os.environ.get("STOQLIB_TEST_PORT") or 0)
    quick = request.config.getvalue("quick_mode")
    quick = quick or _to_falsy(os.environ.get("STOQLIB_TEST_QUICK", None))

    bootstrap_suite(
        address=hostname,
        dbname=dbname,
        port=port,
        username=username,
        password=password,
        quick=quick,
    )

    plugin_cls = request.config.getvalue("plugin_cls") or request.config.inicfg.get("PLUGIN_CLASS")
    if plugin_cls:
        _install_plugin(plugin_cls)



from pytest_stoq.fixtures import *  # noqa


def pytest_addoption(parser):
    group = parser.getgroup("stoq")
    group.addoption(
        "--skip-env-setup",
        action="store_true",
        dest="skip_env_setup",
        default=None,
        help="Skip environment setup. Use with caution.",
    )
    group.addoption(
        "--plugin-cls",
        action="store",
        dest="plugin_cls",
        default=None,
        help=(
            "Specify the module path to the stoq plugin to be installed. "
            "e.g. my_plugin.plugin.PluginClass"
        ),
    )
    group.addoption(
        "--quick",
        action="store_true",
        dest="quick_mode",
        default=None,
        help="Quick database setup mode. Use with caution.",
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import codecs
import os

from setuptools import setup


def read(fname):
    file_path = os.path.join(os.path.dirname(__file__), fname)
    return codecs.open(file_path, encoding="utf-8").read()


setup(
    name="pytest-stoq",
    version="0.4.4",
    author="Stoq Team",
    author_email="stoq-devel@async.com.br",
    maintainer="Stoq Team",
    maintainer_email="stoq-devel@async.com.br",
    license="GNU GPL v3.0",
    url="https://github.com/stoq/pytest-stoq",
    description="A plugin to pytest stoq",
    long_description=read("README.rst"),
    packages=["pytest_stoq"],
    python_requires=">=3.4",
    install_requires=["pytest>=3.5.0"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Framework :: Pytest",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Testing",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    ],
    entry_points={"pytest11": ["stoq = pytest_stoq.plugin"]},
)



from unittest import mock

import pytest
import stoqlib.api
from stoqlib.domain.exampledata import ExampleCreator
from stoqlib.domain.till import Till

from .stoq import _setup_test_environment


@pytest.fixture(autouse=True, scope="session")
def stoq_test_environment(request):
    _setup_test_environment(request)


@pytest.fixture
def store():
    store = stoqlib.api.new_store()
    _close = store.close
    _rollback = store.rollback

    store.close = mock.Mock()
    store.commit = mock.Mock()
    store.rollback = mock.Mock()

    yield store

    store.close = _close
    store.rollback = _rollback
    store.rollback()


@pytest.fixture
def current_station(store):
    return stoqlib.api.get_current_station(store)


@pytest.fixture
def current_user(store):
    return stoqlib.api.get_current_user(store)


@pytest.fixture
def current_branch(store):
    return stoqlib.api.get_current_branch(store)


@pytest.fixture
def current_till(store, example_creator, current_station):
    return Till.get_current(store, current_station) or example_creator.create_till()


@pytest.fixture
def example_creator(store, current_station, current_user, current_branch):
    creator = ExampleCreator()
    creator.set_store(store)
    creator.current_station = current_station
    creator.current_user = current_user
    creator.current_branch = current_branch
    return creator
